Uses Cfg
Uses User

Const MRCVersion = 'Mystic Relay Chat MPL v0.10'
Const CLBuffer	= 300
Const InputSize = 255

Type MRCRec	= Record
	FromUser	: String[30]
	FromSite	: String[30]
	FromChan	: Integer
	ToUser		: String[30]
	ToSite		: String[30]
	ToChan		: Integer
	ChLine		: String[InputSize]
End

Type ActionRec = Record
	ActTag	: String[10]
	ActTo	: String[30]
	ActMe	: String[30]
	ActAll	: String[30]
	ActGen	: String[30]
End

Var ChatLines	: Array [1..CLBuffer] of String[160]
Var WinTL, WinTT, WinBL, WinBB	: Byte
Var WinAttr		: Byte
Var WinSize		: Integer
Var PromptX, PromptY: Byte
Var PromptAttr	: Byte
Var RoomX, RoomY: Byte
Var RoomAttr	: Byte
Var ChanX, ChanY: Byte
Var ChanAttr	: Byte
Var MyNamePrompt: String
Var MyChatRoom	: Integer = 1
Var SiteTag		: String
Var UserTag		: String
Var ChanTag		: String = '1'
Var ServFile	: MRCRec
Var BBSTempStub	: String
Var PInUse		: String
Var ChatSeed	: Integer = ((Random(9)+1)*10000)+((Random(9)+1)*1000)+((Random(9)+1)*100)+((Random(9)+1)*10)+(Random(9)+1)

Include chat.inc

Procedure CleanOutTempDir
Begin
	FindFirst(CfgTempPath+'*.mrc',66)
	While DosError = 0 Do Begin
		If FileExist(CfgTempPath+DirName) Then
			FileErase(CfgTempPath+DirName)
	End
	FindClose
	If FileExist(PInUse) Then
		fileErase(PInUse)
End

Function AmIFirst:Boolean
Var D,X	: Integer
Var Ret	: Boolean = False
Var S	: String
Begin
	D:=0
	For X:=1 To CfgTnNodes Do Begin
		S:=BBSTempStub+Int2Str(X)+PathChar+'tchat.inuse'
		If FileExist(S) And D=0 Then Begin
			D:=X
		End
	End
	If D = NodeNum Then Ret:=True
	AmIFirst:=Ret
End

Procedure UpdateScreen
Begin
	If ReadChat(NodeNum) Then Begin
		If ReadChatRoom(Chat.ChatRoom) Then Begin
			WriteXY(RoomX,RoomY,RoomAttr,PadRt(Int2Str(Chat.ChatRoom),2,' '))
			WriteXY(ChanX,ChanY,ChanAttr,PadRt(Chatroom.Name,78-ChanX,' '))
		End
	End
End

Procedure ShowChat(Top:Integer)
Var C,T,L,Y,X	: Integer
Var G,V,W	: String = ''
Var N2D		: Boolean = True
Begin
	Y:=CLBuffer-WinSize-Top
	While ChatLines[Y]='' And Y < CLBuffer Do
		Y:=Y+1
	For X:=WinTT To WinBB Do Begin
		GoToXy(1,X)
		Write(ChatLines[Y]+'|16|11|$X79 ')
		Y:=Y+1
	End
End

Function ParseChat(S:String) : MrcREc
Var MR	: MrcRec
Begin
	MR.FromUser:=WordGet(1,S,'~')
	MR.FromSite:=WordGet(2,S,'~')
	MR.FromChan:=Str2Int(WordGet(3,S,'~'))
	MR.ToUser:=WordGet(4,S,'~')
	MR.ToSite:=WordGet(5,S,'~')
	MR.ToChan:=Str2Int(WordGet(6,S,'~'))
	MR.ChLine:=WordGet(7,S,'~')
	ParseChat:=MR
End

Procedure RedrawScreen
Begin
	DispFile('ansitele')
	ReadChat(NodeNum)
	UpdateScreen
	ShowChat(0)
End

Procedure Add2Chat(S:String)
Var E,W,L,B,A,X	: Integer
Var S1,S2,S3	: String=''
Begin
	E:=Pos('>',StripMCI(S))+1
	While E>0 Do Begin
		S3:=S3+' '
		E:=E-1
	End
	S1:=S
	Repeat
		B:=Length(S1)
		A:=Length(StripMCI(S1))
		L:=80-(A-B)
		S2:=''
		W:=StrWrap(S1,S2,L)
		For X:=1 To CLBuffer-1 Do 
			ChatLines[X]:=ChatLines[X+1]
		ChatLines[CLBuffer]:=S1
		S1:='|07'+S3+S2
	Until S2=''
End

Procedure MakeChatEntry(S:String)
Var Fil	: String = CfgDataPath+'mrc'+PathChar+Int2Str(ChatSeed)+Int2Str(Random(99))+Int2Str(Random(99))+'.mrc'
Begin
	AppendText(Fil,S)
	ChatSeed:=ChatSeed+1
End

Procedure SendOut(FU,FS,FC,TU,TS,TC,S:String)
Var TX	: String
Begin
	// Format:
	// FromUser~FromSite~FromChannel~ToUser~ToSite~ToChannel~Text
	TX:=FU+'~'+FS+'~'+FC+'~'+TU+'~'+TS+'~'+TC+'~'+S+'~'
	MakeChatEntry(TX)
End

Procedure SendToMe(S:String)
//Var Me	: String = UserTag+','+SiteTag+','+ChanTag+','+UserTag+','+SiteTag+','+ChanTag+','+S
Var Me	: String = 'CLIENT~ALL~0~'+UserTag+'~ALL~0~'+S
Begin
	AppendText(CfgTempPath+'local2me.tmp',Me)
End

Procedure SendToAll(S:String)
Begin
	SendOut(UserTag,SiteTag,ChanTag,'ALL','ALL','0',S)
End

Procedure SendToChannel(S:String)
Begin
	SendOut(UserTag,SiteTag,ChanTag,'ALL','ALL',ChanTag,S)
End

Procedure SendToUser(U,S:String)
Begin
	SendOut(UserTag,SiteTag,ChanTag,U,'ALL','0',S)
End

Procedure SendToClient(S:String)
Begin
	SendOut(UserTag,SiteTag,ChanTag,'CLIENT','ALL','0',S)
End

Procedure ProcessChat(MR:MRCRec)
Var Ok2Send : Boolean = True
Begin
	If MR.ToSite <> 'ALL' And (Upper(MR.ToSite) <> Upper(SiteTag)) Then Ok2Send:=False
	If MR.ToChan <> 0 And ( MR.ToChan <> MyChatRoom ) Then Ok2Send:=False
	If MR.ToUser <> 'ALL' And (Pos(Upper(MR.ToUser),Upper(UserTag))=0 And Length(MR.ToUser)>3)  Then Ok2Send:=False
	If Ok2Send Then 
		Add2Chat(MR.ChLine)
	
End

Procedure ReadChatFiles
Var F1	: File	
Var S	: String
Var Ret	: Boolean = False
Begin
	FindFirst(CfgTempPath+'*.mrc',66)
	While DOSError = 0 Do Begin
		Ret:=True
		fAssign(F1,CfgTempPath+DirName,66)
		fReset(F1)
		While Not fEof(F1) Do Begin
			fReadLn(F1,S)
			ServFile:=ParseChat(S)
			ProcessChat(ServFile)
		End
		fClose(F1)
		fileErase(CfgTempPath+DirName)
		FindNext
	End	
	FindClose
	If Ret Then ShowChat(0)
End

Procedure JoinChannel(S:String)
Var I	: Integer
Begin
	I:=Str2Int(S)
	If ReadChat(NodeNum) Then Begin
		Chat.InChat:=True
		Chat.ChatRoom:=I
		SaveChat(NodeNum)
		UpdateScreen
		MyChatRoom:=I
		ChanTag:=Int2Str(MyChatRoom)
	End	
	SendToAll('|07- '+StripMCI(UserAlias)+' is now chatting in channel '+Int2Str(I))
End

Procedure Init
Var X,Y,MyAttr: Integer
Var K,S		: String = ''
Begin
	BBSTempStub:=CfgTempPath
	Y:=Pos(Int2Str(NodeNum),BBSTempStub) 
	If Y > 0 Then
		Delete(BBSTempStub,Y,Length(Int2Str(NodeNum))+1)
	MyAttr:=NodeNum+1
	If MyAttr > 8 Then MyAttr:=2
	MyNamePrompt:='|'+PadLt(Int2Str(MyAttr),2,'0')
	MyNamePrompt:=MyNamePrompt+'<|'+PadLt(Int2Str(MyAttr+8),2,'0')
	MyNamePrompt:=MyNamePrompt+StripMCI(UserAlias)
	MyNamePrompt:=MyNamePrompt+'|'+PadLt(Int2Str(MyAttr),2,'0')+'> |07'
	DispFile('ansitele')
	GetScreenInfo(1,WinTL,WinTT,WinAttr)
	GetScreenInfo(2,WinBL,WinBB,WinAttr)
	GetScreenInfo(3,PromptX,PromptY,PromptAttr)
	GetScreenInfo(4,RoomX,RoomY,RoomAttr)
	GetScreenInfo(5,ChanX,ChanY,ChanAttr)
	WinSize:=WinBB-WinTT
	ReadChat(NodeNum)
	ShowChat(0)
	SiteTag:=UPPER(MCI2Str('BN'))
	Repeat
		X:=Pos(' ',SiteTag)
		If X > 0 Then Delete(SiteTag,X,1)
	Until X < 1
	UserTag:=StripMCI(Upper(UserAlias))
	PInUse:=CfgTempPath+'tchat.inuse'
	AppendText(PInUse,'0')
	MenuCmd('NA','Mystic Relay Chatting')
End

Procedure DoHelp
Begin
	Write('|16|11')
	DispFile('telehelp')
	RedrawScreen
End

Procedure DoWho
Begin
	Write('|16|11')
	MenuCmd('NW','')
	RedrawScreen
End

Procedure DoCommand(S:String)
Begin
	SendToClient(S)
End

Procedure ChangeTopic(S:String)
Var R	: String
Begin
	R:=Copy(S,8,Length(S)-7)
	If ReadChat(NodeNum) Then Begin
		ChatRoom.Name:=R
		SaveChatRoom(Chat.ChatRoom)
		UpdateScreen
	End
End

Procedure DoPrivateMsg(S:String)
Var M,U	: String
Var L	: Integer
Begin
	U:=Upper(WordGet(2,S,' '))
	L:=Pos(U,Upper(S))
	L:=L+Length(U)+1
	M:='|02<|10'+UserAlias+'|02 private> |07'+Copy(S,L,Length(S)-L+1)
	SendToUser(U,M)
End

Procedure DoBroadcast(S:String)
Var M	: String
Begin
	M:='|05<|13'+UserAlias+'|05 broadcast> |07'+Copy(S,4,Length(S)-3)
	SendToAll(M)
End

Procedure DoMeAction(S:String)
Var R	: String
Begin
	R:=Copy(S,5,Length(S)-4)
	SendToChannel('|13* '+StripMCI(UserAlias)+' ' + R)
End

Function WhoInChat
Var X	: Integer
Var ChatRoomName : String = 'Chat Room'
Var Ptx	: String
Begin
	WriteLn('|16|CL'+GetPrompt(332))
	For X:=1 To CfgTnNodes Do Begin
		If ReadChat(X) Then Begin
			If Chat.Active Then Begin
				If ReadChatRoom(Chat.ChatRoom) Then
					ChatRoomName:=ChatRoom.Name
				Ptx:=GetPrompt(333)
				setpromptinfo(1,Chat.Name)
				setpromptinfo(3,Int2Str(Chat.ChatRoom))
				Setpromptinfo(4,ChatRoomName)
				WriteLn(Ptx)
			End
		End
	End
	Pause
	RedrawScreen
End

Function InputLine:String
Var IX,UL	: Integer
Var Ch	: Char = 'X'
Begin
	UL:=Length(StripMCI(MyNamePrompt))
	IX:=PromptX+Length(StripMCI(MyNamePrompt))
	GoToXy(PromptX,PromptY)
	Write('|16'+MyNamePrompt+'|17|15|$X79 ')
	GoToXy(PromptX,PromptY)
	Write('|16'+MyNamePrompt)
	While Not Keypressed Do Begin
		ReadChatFiles
		Delay(200)
	End
	GoToXy(PromptX,PromptY)
	Write('|16'+MyNamePrompt+'|17|15|$X79 ')
	GoToXy(PromptX,PromptY)
	Write('|16'+MyNamePrompt)

	InputLine:=Input(79-UL,InputSize,11,'')
	GoToXy(PromptX,PromptY)
	Write('|16'+MyNamePrompt+'|17|15|$X79 ')
End

Procedure DoCls
Var X	: Integer 
Begin
	For X:=1 To CLBuffer Do Begin
		ChatLines[X]:=''
	End
	SendToChannel('')
End

Function Highwater:Integer
Var X,Ret	: Integer = 0
Begin
	For X:=1 To CLBuffer Do Begin
		If ChatLines[X] = '' Then
			Ret:=X
	End
	Highwater:=Ret
End

Procedure DoScrollBack
Var Done	: Boolean = False
Var Ch		: Char
Var PR		: String
var HW,LC	: Integer = 0
Begin
	While Not Done Do Begin
		//If LC < 0 Then LC:=0
		//If LC > CLBuffer Then LC:=CLBuffer
		ShowChat(LC)
		GoToXY(PromptX,PromptY)
		Write('|16'+GetPrompt(237))
		HW:=CLBuffer-HighWater
		Ch:=ReadKey
		If IsArrow Then Begin
			Case Ch Of
				#72: If LC < HW Then LC:=LC+1
				#80: If LC > 0 Then LC:=LC-1
			End
		End Else Begin
			Case Ch Of
				#27: Done:=True
				#18: If LC < HW+WinSize Then LC:=LC+WinSize
				#3 : If LC > WinSize Then LC:=LC-WinSize
			End
		End
	End
	ShowChat(0)
End

Procedure ShowAllRooms
Var X	: Integer
Begin
	SendToMe('|16|07- |12Chat Rooms |07-')
	For X:=1 To 99 Do Begin
		If ReadChatRoom(X) Then Begin
			If Upper(ChatRoom.Name) <> Upper('None') Then
				SendToMe('|16|07- |04(|12'+Int2Str(X)+'|04) |12'+Chatroom.Name)
		End
	End	
	If FileExist(CfgTempPath+'local2me.tmp') Then FileCopy(CfgTempPath+'local2me.tmp',CfgTempPath+'local2me.mrc')
	If FileExist(CfgTempPath+'local2me.tmp') Then FileErase(CfgTempPath+'local2me.tmp')
End

Procedure Main
Var Done	: Boolean = False
Var W1,W2,UIL	: String
Var IL		: String
Begin
	UpdateScreen
	Repeat
		IL:=InputLine
		If Pos('/',IL) = 1 Then Begin
			W1:=Upper(WordGet(1,IL,' '))
			W2:=WordGet(2,IL,' ')
			
			Case W1 Of
				'/?'	: DoHelp
				'/B'	: DoBroadcast(IL)
				'/BBSES': DoCommand('CONNECTED')
				'/CHANNEL': DoCommand('CHANNEL')
				'/CHATTING': DoCommand('CHATTING')
				'/CHATTERS': DoCommand('CHATTING')
				'/CLS'	: DoCls
				'/JOIN' : JoinChannel(W2)
				'/ME'   : DoMeAction(IL)
				'/NAMES': WhoInChat
				'/Q'    : Done:=True
				'/ROOMS': ShowAllRooms
				'/SCROLL': DoScrollBack
				'/TOPIC': ChangeTopic(IL)
				'/T','/MSG','/TELL': DoPrivateMsg(IL)
				'/WHO'  : DoWho
				'/WHOON': DoCommand('WHOON')
				'/VERSION': DoCommand('VERSION')
			End
		End Else Begin
			If Length(IL) > 0 Then
				SendToChannel(MyNamePrompt+IL)
		End
	Until Done
End

Begin
	GetThisUser
	Init
	RedrawScreen
	SendToAll('|07- '+UserAlias+' has entered chat')
	JoinChannel(ChanTag)
	DoCommand('CONNECTED')
	DoCommand('WHOON')
	Main
	SendToAll('|07- '+UserAlias+' has left chat')
	Write('|16|11|CL')
	If ReadChat(NodeNum) Then Begin
		Chat.InChat:=False
		Chat.ChatRoom:=1
		SaveChat(NodeNum)
	End
	CleanOutTempDir
End
